package com.localmatters.mobile.weather;

import com.localmatters.mobile.weather.mag.CityInfo;
import com.localmatters.mobile.weather.mag.Forecast;
import com.localmatters.mobile.weather.mag.WeatherListing;
import com.localmatters.mobile.weather.mag.WeatherService;
import com.localmatters.mobile.weather.mag.WeatherServiceSoap;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.xml.namespace.QName;
import javax.xml.ws.WebServiceRef;

/**
 * The weather EJB.
 *
 * @author Daniel Cavalcanti
 */
@Stateless()
public class WeatherBean implements WeatherRemote, WeatherLocal {

    // **********************************************************************
    // Constants
    // **********************************************************************
    
    /**
     * The constant to identify the US country context.
     */
    public static final String US = "United States";
    
    /**
     * The constant to identify the Canada country context.
     */
    public static final String CANADA = "Canada";
    
    // **********************************************************************
    // Instance variables
    // **********************************************************************
    
//    @WebServiceRef(wsdlLocation = "http://webservices.myareaguide.com/weather-new/WeatherService.asmx?WSDL")
//    private WeatherService service;
    
    /**
     * The MAG weather web service port.
     */
    private WeatherServiceSoap port;
    
    /**
     * The QName namespace URI.
     */
    @Resource(name = "qname-namespace-uri")
    private String namespaceURI;
    
    /**
     * The QName local part.
     */
    @Resource(name = "qname-local-part")
    private String localPart;
    
    /**
     * The MAG-WS WSDL URL.
     */
    @Resource(name = "wsdl-url")
    private String wsdlURL;
    
    /**
     * The country context.
     */
    @Resource(name = "country-context")
    private String country;
    
    // **********************************************************************
    // Constructors
    // **********************************************************************
    
    /**
     * Creates a new instance of WeatherBean.
     */
    public WeatherBean() {
    }
    
    /**
     * Establishes connection to MAG-WS.va
     */
    @PostConstruct()
    private void init() {
        
        try {
            WeatherService service = new WeatherService(new URL(wsdlURL), new QName(namespaceURI, localPart));
            port = service.getWeatherServiceSoap();
        } catch (MalformedURLException ex) {
            System.err.println("Error: " + ex.getMessage());
            ex.printStackTrace();
        }
        
    }
    
    // **********************************************************************
    // Weather methods
    // **********************************************************************
    
    /**
     * Returns weather forecast for a location. The location can be a postal
     * code or a city, state string.
     *
     * @param location The location to get weather forecast for.
     * @return The weather forecast.
     */
    public WeatherForecast forecast(String location)
    throws WeatherException {
        
//        // the weather listing
//        WeatherListing listing = null;
//        
//        // get country context dependent listing
//        if (country == null)
//            throw new WeatherException("Country context not defined.");
//        else if (country.equals(US))
//            listing = port.usForecast(location);
//        else if (country.equals(CANADA))
//            listing = port.canadaForecast(location);
//        else
//            throw new WeatherException("Invalid country context");
//        
//        // encapsulate and return listing into weather forecast bean
//        
//        List<LocationDescription> locationDescriptions = getLocationDescriptions(listing);
//        List<ForecastDescription> forecastDescriptions = getForecastDescriptions(listing);
//        
//        return new WeatherForecast(locationDescriptions, forecastDescriptions);
        
        return new WeatherForecast(
            new ArrayList<LocationDescription>(),
            new ArrayList<ForecastDescription>()
        );
        
    }
    
    
    // **********************************************************************
    // Auxiliary methods
    // **********************************************************************
    
    /**
     * Encapsulates the location descriptions in the listing.
     *
     * @param listing The weather listing.
     * @return The location descriptions list.
     */
    private List<LocationDescription> getLocationDescriptions(WeatherListing listing) {
        
        // create location descriptions list
        List<LocationDescription> locationDescriptions = new ArrayList<LocationDescription>(listing.getCity().size());
        
        // populate location descriptions list
        for (CityInfo cityInfo : listing.getCity()) {
            
            LocationDescription locationDescription = new LocationDescription();
            
            locationDescription.setCity(cityInfo.getName());
            locationDescription.setState(cityInfo.getState());
            locationDescription.setPostal(cityInfo.getPostCode());
            locationDescription.setCountry(cityInfo.getCountry());
            
            locationDescriptions.add(locationDescription);
            
        }
        
        // return location descriptions list
        return locationDescriptions;
        
    }
    
    /**
     * Encapsulates the forecast descriptions in the listing.
     *
     * @param listing The weather listing.
     * @return The forecast descriptions list.
     */
    private List<ForecastDescription> getForecastDescriptions(WeatherListing listing) {
        
        // create forecast descriptions list
        List<ForecastDescription> forecastDescriptions = new ArrayList<ForecastDescription>(listing.getFiveDayForecast().getForecast().size());
        
        // populate forecast descriptions list
        for (Forecast forecast : listing.getFiveDayForecast().getForecast()) {
            
            ForecastDescription forecastDescription = new ForecastDescription();
            
            forecastDescription.setDate(forecast.getDate());
            forecastDescription.setDescription(forecast.getDescription());
            forecastDescription.setHigh(forecast.getHigh());
            forecastDescription.setLow(forecast.getLow());
            
            forecastDescriptions.add(forecastDescription);
            
        }
        
        // return forecast descriptions list
        return forecastDescriptions;
        
    }
    
}
